# Copyright (C) 2015 by Per Unneberg
from bioodo import samtools, odo, DataFrame
from pytest_ngsfixtures.config import application_fixtures
import utils

fixtures = application_fixtures(application="samtools")

stat_fixtures = [x for x in fixtures if x[1] == "samtools_stats"]
samtools_stats = utils.fixture_factory(stat_fixtures)

idxstats_fixtures = [x for x in fixtures if x[1] == "samtools_idxstats"]
samtools_idxstats = utils.fixture_factory(idxstats_fixtures)


def test_basic_statistics(samtools_stats):
    _stats = {'1.2': {'se': 60037, 'pe': 120110},
              '1.3.1': {'se': 60000, 'pe': 120000},
              '1.4.1': {'se': 60000, 'pe': 120000}}
    module, command, version, end, pdir = samtools_stats
    fn = str(pdir.join("medium.stats.txt"))
    df = odo(samtools.resource_samtools_stats(fn), DataFrame)
    assert (list(df.index)[0] == 'raw total sequences')
    assert(df.loc["sequences", "value"] == _stats[version][end])


def test_GCC(samtools_stats):
    _gcc_stats = {'1.2': {'se': 30.12, 'pe': 30.21},
                  '1.3.1': {'se': 30.19, 'pe': 30.27},
                  '1.4.1': {'se': 30.19, 'pe': 30.27}}
    module, command, version, end, pdir = samtools_stats
    fn = str(pdir.join("medium.stats.txt"))
    df = odo(samtools.resource_samtools_stats(fn, key="GCC"), DataFrame)
    assert (df.loc[1]["A"] == _gcc_stats[version][end])


def test_FFQ(samtools_stats):
    _ffq_stats = {'1.2': {'se': 27624, 'pe': 27630},
                  '1.3.1': {'se': 27598, 'pe': 27598},
                  '1.4.1': {'se': 27598, 'pe': 27598}}
    module, command, version, end, pdir = samtools_stats
    fn = str(pdir.join("medium.stats.txt"))
    df = odo(samtools.resource_samtools_stats(fn, key="FFQ"), DataFrame)
    assert (df.loc[1][33] == _ffq_stats[version][end])


def test_idxstats(samtools_idxstats):
    module, command, version, end, pdir = samtools_idxstats
    fn = str(pdir.join("medium.idxstats.txt"))
    df = odo(fn, DataFrame)
    assert (df.loc[0][0] == "scaffold1")



# Copyright (C) 2015 by Per Unneberg
import re
import pandas as pd
import bioodo
from bioodo import resource, annotate_by_uri, pivot, DataFrame, utils
import logging


logger = logging.getLogger(__name__)
config = bioodo.__RESOURCE_CONFIG__['qualimap']

COVERAGE_PER_CONTIG_COLUMNS = ["chr", "chrlen", "mapped_bases",
                               "mean_coverage", "sd"]


re_trim = re.compile(r'(,|\s+bp.*$|\s+\(.*%\)|%)')


def _split_x(x, delim=" = "):
    y = x.strip().split(delim)
    return [y[0], re_trim.sub("", y[1])]


@resource.register(config['genome_results']['pattern'],
                   priority=config['genome_results']['priority'])
@pivot
@annotate_by_uri
def resource_genome_results(uri, key="Globals", **kwargs):
    with open(uri) as fh:
        data = "".join(fh)
    sections = re.split(">+\s+[a-zA-Z ]+", data)
    section_names = ["Header"] + [
        re.sub(" ", "_", x) for x in re.findall(">+\s+([a-zA-Z ]+)", data)]
    d = dict()
    for h, sec in zip(section_names, sections):
        if h == "Coverage_per_contig":
            d[h] = DataFrame.from_records(
                [re.split("\s+", x.strip()) for x in sec.split("\n") if x],
                columns=COVERAGE_PER_CONTIG_COLUMNS,
                index="chr")
            d[h] = d[h].apply(pd.to_numeric)
        elif h in ["Coverage", "Header"]:
            pass
        else:
            d[h] = DataFrame.from_records(
                [_split_x(x) for x in sec.split("\n") if x],
                columns=["statistic", "value"],
                index="statistic")
            if h not in ["Input"]:
                d[h] = d[h].apply(pd.to_numeric)
    return d[key]


@resource.register(config['data_frame']['pattern'],
                   priority=config['data_frame']['priority'])
@pivot
@annotate_by_uri
def resource_read_data_frame(uri, **kwargs):
    d = pd.read_table(uri)
    columns = list(d.columns)
    columns[0] = re.sub("#", "", columns[0]).strip()
    d.columns = columns
    return d


aggregate = utils.aggregate_factory("qualimap")



# Copyright (C) 2015 by Per Unneberg
import pandas as pd
import bioodo
from bioodo import resource, annotate_by_uri, pivot, utils

config = bioodo.__RESOURCE_CONFIG__['rpkmforgenes']


@resource.register(config['rpkmforgenes']['pattern'],
                   priority=config['rpkmforgenes']['priority'])
@pivot
@annotate_by_uri
def resource_rpkmforgenes(uri, **kwargs):
    with open(uri):
        data = pd.read_csv(uri, sep="\t", header=None, comment="#",
                           names=["gene_id", "transcript_id", "FPKM", "TPM"],
                           index_col=["gene_id"])
    return data


aggregate = utils.aggregate_factory("rpkmforgenes")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from collections import Mapping


def update_config(config, overwrite_config):
    """Recursively update dictionary config with overwrite_config.

    See
    http://stackoverflow.com/questions/3232943/update-value-of-a-nested-dictionary-of-varying-depth
    for details.

    Args:
      config (dict): dictionary to update
      overwrite_config (dict): dictionary whose items will overwrite
                               those in config

    """

    def _update(d, u):
        for (key, value) in u.items():
            if (isinstance(value, Mapping)):
                d[key] = _update(d.get(key, {}), value)
            else:
                d[key] = value
        return d

    _update(config, overwrite_config)



# Copyright (C) 2016 by Per Unneberg
import logging
import pandas as pd
from bioodo import resource, annotate_by_uri, pivot, DataFrame, utils
import bioodo

logger = logging.getLogger(__name__)
config = bioodo.__RESOURCE_CONFIG__['samtools']


SECTION_NAMES = ['SN', 'FFQ', 'LFQ', 'GCF', 'GCL', 'GCC',
                 'IS', 'RL', 'ID', 'IC', 'COV', 'GCD']
COLUMNS = {
    'SN': ['statistic', 'value'],
    'FFQ': None,
    'LFQ': None,
    'GCF': ['percent', 'count'],
    'GCL': ['percent', 'count'],
    'GCC': ['cycle', 'A', 'C', 'G', 'T', 'ACGT_PCT', 'NO_PCT'],
    'IS': ['insert_size', 'pairs_total', 'inward_oriented_pairs',
           'outward_oriented_pairs', 'other_pairs'],
    'RL': ['length', 'count'],
    'ID': ['length', 'insertions', 'deletions'],
    'IC': ['cycle', 'insertions_fwd', 'insertions_rev',
           'deletions_fwd', 'deletions_rev'],
    'COV': ['bin', 'coverage', 'count'],
    'GCD': ['percent', 'unique', 'p10', 'p25', 'p50', 'p75', 'p90'],
}

IDXSTATS_COLUMNS = ["name", "length", "mapped", "unmapped"]


@resource.register(config['stats']['pattern'],
                   priority=config['stats']['priority'])
@pivot
@annotate_by_uri
def resource_samtools_stats(uri, key="SN", **kwargs):
    """Parse samtools stats text output file.

    Args:
      uri (str): filename
      key (str): result section to return

    Returns:
      DataFrame: DataFrame for requested section
    """
    def _parse():
        data = []
        with open(uri) as fh:
            for x in fh.readlines():
                if not x.startswith(key):
                    continue
                y = []
                for z in x.replace(":", "").strip().split("\t")[1:]:
                    if not z.startswith("#"):
                        y.append(z)
                data.append(y)
        return data

    if key not in SECTION_NAMES:
        raise KeyError("Not in allowed section names; " +
                       "allowed values are {}".format(
                           ", ".join(SECTION_NAMES + ["Summary"])))
    data = _parse()
    if key in ['FFQ', 'LFQ']:
        df = DataFrame.from_records(data)
        df = df.apply(pd.to_numeric, errors='ignore')
        df.columns = ['cycle'] + [str(x) for x in range(len(df.columns) - 1)]
        df = df.set_index('cycle')
    else:
        n = len(data[0])
        df = DataFrame.from_records(data, columns=COLUMNS[key][0:n])
        df = df.apply(pd.to_numeric, errors='ignore')
        df = df.set_index(df[COLUMNS[key][0]])
        del df[COLUMNS[key][0]]
    return df


@resource.register(config['idxstats']['pattern'],
                   priority=config['idxstats']['priority'])
@annotate_by_uri
def resource_samtools_idxstats(uri, **kwargs):
    """Parse samtools idxstats output file.

    From the documentation:

        The output is TAB-delimited with each line consisting of
        reference sequence name, sequence length, # mapped reads and #
        unmapped reads.

    Args:
      uri (str): filename

    Returns:
      DataFrame:

    """
    df = pd.read_table(uri, names=IDXSTATS_COLUMNS)
    return df


aggregate = utils.aggregate_factory("samtools")



# Copyright (C) 2015 by Per Unneberg
from bioodo import picard, odo, DataFrame
from pytest_ngsfixtures.config import application_fixtures
import utils

fixtures = application_fixtures(application="picard")
insert_metrics = utils.fixture_factory(
    [x for x in fixtures if "CollectInsertSizeMetrics" in x[1]])
align_metrics = utils.fixture_factory(
    [x for x in fixtures if "CollectAlignmentSummaryMetrics" in x[1]])
aggregate_data_insert = utils.aggregation_fixture_factory(
    [x for x in fixtures if "CollectInsertSizeMetrics" in x[1]], 2)


def test_hist_metrics(insert_metrics):
    module, command, version, end, pdir = insert_metrics
    fn = pdir.join("medium.insert_size_metrics")
    metrics = odo(str(fn), DataFrame)
    hist = odo(str(fn), DataFrame, key="hist")
    assert all(metrics["MEDIAN_INSERT_SIZE"] == [367])
    assert all(hist["insert_size"][0:3] == [19, 22, 23])


def test_metrics(align_metrics):
    module, command, version, end, pdir = align_metrics
    fn = pdir.join("medium.align_metrics")
    metrics = odo(str(fn), DataFrame)
    if end == "pe":
        assert metrics.loc["FIRST_OF_PAIR"]["MEAN_READ_LENGTH"] - 92.29 < 0.01
    else:
        assert metrics.loc["UNPAIRED"]["MEAN_READ_LENGTH"] - 92.29975 < 0.001


def test_aggregate_insert_data(aggregate_data_insert):
    module, command, version, end, pdir = aggregate_data_insert
    df = picard.aggregate(
        [str(x.listdir()[0]) for x in pdir.listdir() if x.isdir()],
        regex=".*/(?P<repeat>[0-9]+)/medium.insert_size_metrics")
    assert sorted(list(df["repeat"].unique())) == ['0', '1']



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from bioodo import bamtools, odo, DataFrame
from pytest_ngsfixtures.config import application_fixtures
import utils

fixtures = application_fixtures(application="bamtools")
bamtools_data = utils.fixture_factory([x for x in fixtures])
bamtools_aggregate_data = utils.aggregation_fixture_factory(
    [x for x in fixtures], 2)


def test_bamtools(bamtools_data):
    module, command, version, end, pdir = bamtools_data
    df = odo(str(pdir.listdir()[0]), DataFrame)
    n = 59499 if end == "se" else 119413
    assert df.loc["Mapped reads", "value"] == n


def test_bamtools_aggregate(bamtools_aggregate_data):
    module, command, version, end, pdir = bamtools_aggregate_data
    df = bamtools.aggregate(
        [str(x.listdir()[0]) for x in pdir.listdir() if x.isdir()],
        regex=".*/(?P<repeat>[0-9]+)/medium.stats")
    assert sorted(list(df["repeat"].unique())) == ['0', '1']


def test_bamtools_pivot(bamtools_data):
    module, command, version, end, pdir = bamtools_data
    df = odo(
        str(pdir.listdir()[0]), DataFrame,
        values=["value", "percent"], columns="statistic",
        index="sample",
        regex=".*/(?P<sample>medium.*)"
    )
    n = 59499 if end == "se" else 119413
    assert df["value", "Mapped reads"].loc["medium.stats"] == n


def test_bamtools_aggregate_pivot(bamtools_aggregate_data):
    module, command, version, end, pdir = bamtools_aggregate_data
    df = bamtools.aggregate(
        [str(x.listdir()[0]) for x in pdir.listdir() if x.isdir()],
        values=["value", "percent"], columns="statistic",
        index="repeat",
        regex=".*/(?P<repeat>[0-9]+)/medium.stats"
    )
    n = 59499 if end == "se" else 119413
    assert df["value", "Mapped reads"].loc["0"] == n



# Copyright (C) 2015 by Per Unneberg
from bioodo import qualimap, odo, DataFrame
from pytest_ngsfixtures.config import application_fixtures
import utils


fixtures = application_fixtures(application="qualimap")

# Separate pe and se cases
outputs = {'pe': {}, 'se': {}}
newfixtures = []
for x in fixtures:
    for k, v in x[4].items():
        y = tuple([x[0], x[1], x[2], x[3], {k: v}])
        newfixtures.append(y)

data = utils.fixture_factory(newfixtures, scope="function", unique=True)
qualimap_aggregate_data = utils.aggregation_fixture_factory(
    [x for x in newfixtures if "homopolymer_indels" in x[4].keys()], 2)


def test_qualimap(data):
    module, command, version, end, pdir = data
    if command.startswith("qualimap_bamqc_genome_results"):
        fn = pdir.listdir()[0]
        df = odo(str(fn), DataFrame, key='Coverage_per_contig')
        assert list(df.columns) == ['chrlen', 'mapped_bases',
                                    'mean_coverage', 'sd']
        assert list(df.index)[0] == 'scaffold1'
    else:
        fn = pdir.listdir()[0]
        df = odo(str(fn), DataFrame)
        assert "#" not in df.columns[0]


def test_qualimap_aggregate(qualimap_aggregate_data):
    module, command, version, end, pdir = qualimap_aggregate_data
    df = qualimap.aggregate(
        [str(x.listdir()[0]) for x in pdir.listdir()
         if x.isdir()],
        regex=".*/(?P<repeat>[0-9]+)/homopolymer_indels.txt")
    assert sorted(list(df["repeat"].unique())) == ['0', '1']


def test_qualimap_aggregate_compress(qualimap_aggregate_data):
    module, command, version, end, pdir = qualimap_aggregate_data
    qualimap.aggregate(
        [str(x.listdir()[0]) for x in pdir.listdir()
         if x.isdir()],
        regex=".*/(?P<repeat>[0-9]+)/homopolymer_indels.txt",
        outfile=str(pdir.join("test.csv.gz")))
    assert pdir.join("test.csv.gz").exists()



# Copyright (C) 2016 by Per Unneberg
import re
import logging
import bioodo
from bioodo import resource, annotate_by_uri, pivot, DataFrame, utils
import pandas as pd

logger = logging.getLogger(__name__)

config = bioodo.__RESOURCE_CONFIG__['bwa']


@resource.register(config['bwa_mem']['pattern'], config['bwa_mem']['priority'])
@pivot
@annotate_by_uri
def resource_bwa_mem(uri, **kwargs):
    """Parse bwa mem log output file.

    Args:
      uri (str): filename

    Returns:
      DataFrame: DataFrame for requested section
    """
    logger.debug("Parsing {} in resource_bwa_preprocess".format(uri))
    with open(uri) as fh:
        data = "".join(fh)
    sections = re.split("Preprocess stats:\n", data)
    parameters = [["parameter"] +
                  [x.strip() for x in y.strip().split(":")]
                  for y in re.sub("Parameters:\n", "", sections[0]).split("\n")
                  if ":" in y]
    preprocess = [["preprocess stats"] +
                  [x.strip() for x in y.strip().split(":")]
                  for y in re.sub("\([0-9\.]+\)", "", sections[1]).split("\n")
                  if y and "wall" not in y]
    df = DataFrame.from_records(parameters + preprocess,
                                columns=["type", "statistic", "value"])
    df = df.set_index('statistic')
    df["value"] = df["value"].apply(pd.to_numeric, errors="ignore")
    return df


aggregate = utils.aggregate_factory("bwa")



import os
import pytest
from pytest_ngsfixtures import factories
import logging


logger = logging.getLogger(__name__)


def fixture_factory(fixture_list, unique=False, **kwargs):
    """Fixture factory to generate fixtures from
    a list of pytest-ngsfixtures fixtures

    Params:
      fixture_list(list): list of tuples, where each tuple consists of
                          module, command, version, end and file name format
      unique(boolean): make unique fixture directories; useful if one
                       wants to separate multiple outputs into
                       separate directories
      kwargs(dict): keyword arguments
    """
    if unique:
        for i in range(len(fixture_list)):
            y = list(fixture_list[i])
            y[1] = "{}_{}".format(y[1], i)
            fixture_list[i] = tuple(y)

    @pytest.fixture(scope="session", autouse=False, params=fixture_list,
                    ids=["{} {}:{}/{}".format(x[0], x[1], x[2], x[3])
                         for x in fixture_list])
    def bioodo_fixture(request, tmpdir_factory):
        # NB: fmtdict is a dictionary, potentially pointing to
        # multiple output files
        module, command, version, end, fmtdict = request.param
        params = {'version': version, 'end': end}
        # Generate pytest_ngsfixtures application output names relative to
        # applications/module directory
        outputs = [fmt.format(**params) for fmt in fmtdict.values()]
        # Add applications/module prefix
        sources = [os.path.join("applications", module, output)
                   for output in outputs]
        # Extract source basenames
        dests = [os.path.basename(src) for src in sources]
        # Generate a unique test output directory name
        fdir = os.path.join(module, str(version), command, end)
        # Make a temporary directory using unique test directory name
        pdir = factories.safe_mktemp(tmpdir_factory, fdir)
        # Symlink pytest_ngsfixtures files to temporary directory; the
        # safe_symlink function automagically uses pytest_ngsfixtures
        # installation directory to infer location of src
        for src, dst in zip(sources, dests):
            factories.safe_symlink(pdir, src, dst)
        return module, command, version, end, pdir
    return bioodo_fixture


def aggregation_fixture_factory(fixture_list, repeat, **kwargs):
    """Fixture factory for aggregation data.

    Aggregation data requires multiple inputs. This can be
    accomplished by using the same output multiple times, generating
    unique fixture names on the fly.

    Params:
      fixture_list (list): an application fixture
      repeat (int): number of times to repeat input file
      kwargs (dict): keyword arguments

    Returns:
       a fixture, corresponding to a directory where input files reside
       in subdirectories named after version and end

    """
    @pytest.fixture(scope=kwargs.get("scope", "session"),
                    autouse=kwargs.get("autouse", False),
                    params=fixture_list,
                    ids=["{} {}:{}/{}".format(x[0], x[1], x[2], x[3])
                         for x in fixture_list])
    def bioodo_aggregation_fixture(request, tmpdir_factory):
        # NB: fmtdict is a dictionary, potentially pointing to
        # multiple output files; if so the multiple files must be put
        # in the same directory
        module, command, version, end, fmtdict = request.param
        params = {'version': version, 'end': end}
        keys = kwargs.get("keys", fmtdict.keys())
        outputs = [fmtdict[k].format(**params)
                   for k in keys] * len(range(repeat))
        # Add applications/module prefix
        sources = [os.path.join("applications", module, output)
                   for output in outputs]
        # Extract source basenames
        dests = [os.path.basename(src) for src in outputs]
        # Generate a unique test output directory
        fdir = os.path.join(module, str(version), command, end)
        # Make a temporary directory using unique test directory name
        pdir = factories.safe_mktemp(tmpdir_factory, fdir)
        # Symlink pytest_ngsfixtures files to temporary directory; the
        # safe_symlink function automagically uses pytest_ngsfixtures
        # installation directory to infer location of src
        count = 0
        for src, dst in zip(sources, dests):
            rdir = pdir.mkdir(str(count))
            factories.safe_symlink(rdir, src, dst)
            count += 1
        return module, command, version, end, pdir
    return bioodo_aggregation_fixture



# Copyright (C) 2015 by Per Unneberg
from bioodo import cutadapt, odo, DataFrame
from pytest_ngsfixtures.config import application_fixtures
import utils

fixtures = application_fixtures(application="cutadapt")
cutadapt_metrics = utils.fixture_factory(fixtures)
cutadapt_aggregate_data = utils.aggregation_fixture_factory(
    [x for x in fixtures], 2)


def test_cutadapt(cutadapt_metrics):
    module, command, version, end, pdir = cutadapt_metrics
    fn = str(pdir.join("cutadapt_metrics.txt"))
    df = odo(fn, DataFrame)
    if end == "se":
        assert df.loc["Reads with adapters"]["value"] == 792
    elif end == "pe":
        assert df.loc["Read 1 with adapter"]["value"] == 792


def test_cutadapt_aggregate(cutadapt_aggregate_data):
    module, command, version, end, pdir = cutadapt_aggregate_data
    df = cutadapt.aggregate(
        [str(x.listdir()[0]) for x in pdir.listdir() if x.isdir()],
        regex=".*/(?P<repeat>[0-9]+)/cutadapt_metrics.txt")
    assert sorted(list(df["repeat"].unique())) == ['0', '1']



# Copyright (C) 2016 by Per Unneberg
import logging
import pandas as pd
import re
import bioodo
from bioodo import resource, annotate_by_uri, pivot, DataFrame, utils

logger = logging.getLogger(__name__)
config = bioodo.__RESOURCE_CONFIG__['bamtools']


@resource.register(config['stats']['pattern'],
                   priority=config['stats']['priority'])
@pivot
@annotate_by_uri
def resource_bamtools_stats(uri, **kwargs):
    """Parse bamtools stats text output file.

    Args:
      uri (str): filename

    Returns:
      DataFrame: DataFrame for requested section
    """
    def _parse():
        data = []
        with open(uri) as fh:
            for x in fh.readlines()[5:]:
                if x.startswith("\n"):
                    continue
                x = re.sub("Read (\d+)", "Read_\\1", x)
                x = re.sub("(^\t|:|'|\s+$)", "", x)
                x = re.sub("\s+(\d+)", "\t\\1", x).split("\t")
                data.append(x)
        return data

    data = _parse()
    df = DataFrame.from_records(data)
    df.columns = ["statistic", "value", "percent"]
    df['percent'].replace("[\(\)%]", "", inplace=True, regex=True)
    df["percent"] = pd.to_numeric(df['percent'], errors="ignore")
    df["value"] = pd.to_numeric(df['value'], errors="ignore")
    df.set_index("statistic", inplace=True, drop=False)
    return df


aggregate = utils.aggregate_factory("bamtools")



# Copyright (C) 2016 by Per Unneberg
import os
import logging
import bioodo
from bioodo import settings

settings.CONFIGFILES = [os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                     ".bioodo.yaml"))]
bioodo.load_config()

pytest_plugins = 'pytester'

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


blacklist = ['__pycache__', '__init__.py', 'tests', '_version.py',
             'pandas.py', 'utils.py']


def pytest_addoption(parser):
    group = parser.getgroup("bioodo", "bioodo test options")
    group.addoption("-M", "--bioodo-module", action="store",
                    default=False,
                    help="module to test",
                    dest="module")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from bioodo import picard
from pytest_ngsfixtures import factories
import pytest


FILES = ["applications/picard/2.9.0/pe/medium.align_metrics"]
data = factories.fileset(src=FILES, dst=["medium.align_metrics.foo"],
                         fdir="data")


def test_parse_uri_failure(data):
    infiles = [str(x) for x in data.listdir()]
    with pytest.raises(NotImplementedError):
        picard.aggregate(infiles)


def test_parse_uri_with_parser(data):
    infiles = [str(x) for x in data.listdir()]
    picard.aggregate(infiles, parser=picard.resource_align_metrics)



# Copyright (C) 2015 by Per Unneberg
import pytest
from bioodo import fastqc, odo, DataFrame

from pytest_ngsfixtures.config import application_fixtures
import utils

fixtures = application_fixtures(application="fastqc")
fastqc_data = utils.fixture_factory(fixtures)
fastqc_aggregate_data = utils.aggregation_fixture_factory(
    [x for x in fixtures], 2)


def test_basic_statistics(fastqc_data):
    module, command, version, end, pdir = fastqc_data
    fn = str(pdir.join("medium_fastqc.zip"))
    df = odo(fn, DataFrame)
    major, minor, patch = version.split(".")
    if int(minor) >= 11:
        assert(list(df.index) == ['Filename', 'File type', 'Encoding',
                                  'Total Sequences',
                                  'Sequences flagged as poor quality',
                                  'Sequence length', '%GC'])
    else:
        assert(list(df.index) == ['Filename', 'File type', 'Encoding',
                                  'Total Sequences', 'Filtered Sequences',
                                  'Sequence length', '%GC'])
    assert(df.loc["Filename", "Value"] == "medium.bam")


def test_summary(fastqc_data):
    module, command, version, end, pdir = fastqc_data
    fn = str(pdir.join("medium_fastqc.zip"))
    df = odo(fn, DataFrame, key="Summary")
    assert(df.loc['Basic_Statistics', 'Value'] == "pass")


def test_wrong_key(fastqc_data):
    module, command, version, end, pdir = fastqc_data
    fn = str(pdir.join("medium_fastqc.zip"))
    with pytest.raises(KeyError):
        odo(fn, DataFrame, key="foo")


def test_per_base_sequence_quality(fastqc_data):
    module, command, version, end, pdir = fastqc_data
    fn = str(pdir.join("medium_fastqc.zip"))
    df = odo(fn, DataFrame, key="Per_base_sequence_quality")
    major, minor, patch = version.split(".")
    if int(minor) <= 10:
        assert df.shape[0] == 28
    else:
        assert df.shape[0] == 55
    assert df.shape[1] == 6


def test_fastqc_aggregate(fastqc_aggregate_data):
    module, command, version, end, pdir = fastqc_aggregate_data
    df = fastqc.aggregate([str(x.listdir()[0]) for x in pdir.listdir()
                           if x.isdir()],
                          regex=".*/(?P<repeat>[0-9]+)/medium_fastqc.zip")
    assert sorted(list(df["repeat"].unique())) == ['0', '1']



# Copyright (C) 2015 by Per Unneberg
from bioodo import vsearch, odo, DataFrame
from pytest_ngsfixtures.config import application_fixtures
import utils

blacklist = ['vsearch_fastqc_filter']
fixtures = [x for x in application_fixtures(application="vsearch")
            if x[1] not in blacklist]
data = utils.fixture_factory(fixtures)
aggregate_data = utils.aggregation_fixture_factory(
    fixtures, 2)


def test_vsearch_fastq_stats(data):
    module, command, version, end, pdir = data
    fn = pdir.join("medium.fastq_stats.txt")
    df = odo(str(fn), DataFrame)
    assert list(df.columns) == ["N", "Pct", "AccPct"]
    assert df.index.name == "L"
    df = odo(str(fn), DataFrame, key="Truncate at first Q")
    assert list(df.columns) == ["Q=5", "Q=10", "Q=15", "Q=20"]
    assert df.index.name == "Len"


def test_vsearch_aggregate(aggregate_data):
    module, command, version, end, pdir = aggregate_data
    df = vsearch.aggregate(
        [str(x.listdir()[0]) for x in pdir.listdir()
         if x.isdir()],
        regex=".*/(?P<repeat>[0-9]+)/medium.fastq_stats.txt")
    assert sorted(list(df["repeat"].unique())) == ['0', '1']



# Copyright (C) 2016 by Per Unneberg
import logging
import pandas as pd
import bioodo
from bioodo import resource, annotate_by_uri, pivot, utils

logger = logging.getLogger(__name__)
config = bioodo.__RESOURCE_CONFIG__['mapdamage']


@resource.register(config['runtime']['pattern'],
                   priority=config['runtime']['priority'])
@pivot
@annotate_by_uri
def resource_mapdamage_runtime(uri, **kwargs):
    """Parse mapdamage runtime log.

    Args:
      uri (str): filename

    Returns:
      DataFrame: DataFrame representation of runtime log
    """
    df = pd.read_table(uri, sep="\t", header=None)
    return df


@resource.register(config['3pGtoA_freq']['pattern'],
                   priority=config['3pGtoA_freq']['priority'])
@pivot
@annotate_by_uri
def resource_mapdamage_3pGtoA_freq(uri, **kwargs):
    """Parse mapdamage 3pGtoA_freq.txt

    Args:
      uri (str): filename

    Returns:
      DataFrame: DataFrame representation of 3pGtoA
    """
    df = pd.read_table(uri, index_col=0)
    return df


@resource.register(config['5pCtoT_freq']['pattern'],
                   priority=config['5pCtoT_freq']['priority'])
@pivot
@annotate_by_uri
def resource_mapdamage_5pCtoT_freq(uri, **kwargs):
    """Parse mapdamage 5pCtoT_freq.txt

    Args:
      uri (str): filename

    Returns:
      DataFrame: DataFrame representation of 5pCtoT
    """
    df = pd.read_table(uri, index_col=0)
    return df


@resource.register(config['mcmc_correct_prob_freq']['pattern'],
                   priority=config['mcmc_correct_prob_freq']['priority'])
@pivot
@annotate_by_uri
def resource_mapdamage_mcmc_correct_prob_freq(uri, **kwargs):
    """Parse mapdamage Stats_out_MCMC_correct_prob.csv

    Args:
      uri (str): filename

    Returns:
      DataFrame: DataFrame representation of mcmc correct prob.
    """
    df = pd.read_csv(uri, index_col="Position")
    del df["Unnamed: 0"]
    return df


@resource.register(config['mcmc_iter']['pattern'],
                   priority=config['mcmc_iter']['priority'])
@pivot
@annotate_by_uri
def resource_mapdamage_mcmc_iter(uri, **kwargs):
    """Parse mapdamage Stats_out_MCMC_iter.csv

    Args:
      uri (str): filename

    Returns:
      DataFrame: DataFrame representation of mcmc iter.
    """
    df = pd.read_csv(uri, index_col=0)
    return df


@resource.register(config['mcmc_iter_summ']['pattern'],
                   priority=config['mcmc_iter_summ']['priority'])
@pivot
@annotate_by_uri
def resource_mapdamage_mcmc_iter_summ_stat(uri, **kwargs):
    """Parse mapdamage Stats_out_MCMC_iter_summ_stat.csv

    Args:
      uri (str): filename

    Returns:
      DataFrame: DataFrame representation of mcmc iter summ stat.
    """
    df = pd.read_csv(uri, index_col=0)
    return df


@resource.register(config['dnacomp']['pattern'],
                   priority=config['dnacomp']['priority'])
@pivot
@annotate_by_uri
def resource_mapdamage_dnacomp(uri, **kwargs):
    """Parse mapdamage dnacomp.txt

    Args:
      uri (str): filename

    Returns:
      DataFrame: DataFrame representation of dnacomp
    """
    df = pd.read_table(uri, comment="#")
    return df


@resource.register(config['dnacomp_genome']['pattern'],
                   priority=config['dnacomp_genome']['priority'])
@pivot
@annotate_by_uri
def resource_mapdamage_dnacomp_genome(uri, **kwargs):
    """Parse mapdamage dnacomp_genome.csv

    Args:
      uri (str): filename

    Returns:
      DataFrame: DataFrame representation of dnacomp_genome
    """
    df = pd.read_csv(uri)
    return df


@resource.register(config['lgdistribution']['pattern'],
                   priority=config['lgdistribution']['priority'])
@pivot
@annotate_by_uri
def resource_mapdamage_lgdistribution(uri, **kwargs):
    """Parse mapdamage lgdistribution.txt

    Args:
      uri (str): filename

    Returns:
      DataFrame: DataFrame representation of lgdistribution
    """
    df = pd.read_table(uri, sep="\s+", comment="#")
    return df


@resource.register(config['misincorporation']['pattern'],
                   priority=config['misincorporation']['priority'])
@pivot
@annotate_by_uri
def resource_mapdamage_misincorporation(uri, **kwargs):
    """Parse mapdamage misincorporation.txt

    Args:
      uri (str): filename

    Returns:
      DataFrame: DataFrame representation of misincorporation
    """
    df = pd.read_table(uri, sep="\t", comment="#")
    return df


aggregate = utils.aggregate_factory("mapdamage")



# Copyright (C) 2015 by Per Unneberg
import pandas as pd
from blaze import resource


@resource.register('.+\.(xls)(.gz)?')
def resource_xls(uri, **kwargs):
    delimiter = "\t"
    if 'delimiter' in kwargs:
        delimiter = kwargs.pop("delimiter")
    if 'dshape' in kwargs:
        kwargs.pop("dshape")
    return pd.read_csv(uri, delimiter=delimiter, **kwargs)



# Copyright (C) 2015 by Per Unneberg
import os
from bioodo import rseqc, odo, DataFrame
from pytest_ngsfixtures.config import application_fixtures
import utils

blacklist = ["rseqc_junction_annotation", "rseqc_read_duplication"]
fixtures = application_fixtures(application="rseqc")
fixture_list = [f for f in fixtures if f[1] not in blacklist]
data = utils.fixture_factory(fixture_list, scope="function")
rseqc_aggregate_data = utils.aggregation_fixture_factory(
    [x for x in fixture_list], 2)


def test_rseqc_parse(data):
    module, command, version, end, pdir = data
    fn = pdir.listdir()[0]
    if command == "rseqc_read_duplication":
        odo(str(fn), DataFrame)
        fn = pdir.listdir()[1]
        odo(str(fn), DataFrame)
    else:
        odo(str(fn), DataFrame)


def test_rseqc_aggregate(rseqc_aggregate_data):
    module, command, version, end, pdir = rseqc_aggregate_data
    infiles = [str(x.listdir()[0]) for x in pdir.listdir() if x.isdir()]
    df = rseqc.aggregate(
        infiles,
        regex=".*/(?P<repeat>[0-9]+)/" + os.path.basename(infiles[0]))
    assert sorted(list(df["repeat"].unique())) == ['0', '1']



# Copyright (C) 2015 by Per Unneberg
import os
from blaze import DataFrame, odo, resource
import pandas as pd
from bioodo.pandas import annotate_by_uri, pivot
import pytest


@resource.register('.+\.csv', priority=20)
@pivot
@annotate_by_uri
def resource_csv_to_df(uri, **kwargs):
    df = pd.read_csv(uri)
    return df


@pytest.fixture(scope="module")
def dataframe1(tmpdir_factory):
    fn = tmpdir_factory.mktemp('data').join('sample1.dataframe1.csv')
    fn.write("""foo,bar\n1,2\n3,4""")
    return fn


@pytest.fixture(scope="module")
def dataframe2(tmpdir_factory):
    fn = tmpdir_factory.mktemp('data').join('sample2.dataframe2.csv')
    fn.write("""foo,bar\n5,6\n7,8""")
    return fn


def test_annotate_df(dataframe1, dataframe2):
    df1 = odo(str(dataframe1), DataFrame, annotate=True)
    df2 = df1.append(odo(str(dataframe2), DataFrame, annotate=True))
    assert set([os.path.basename(x) for x in df2['uri']]) == \
        {'sample1.dataframe1.csv', 'sample2.dataframe2.csv'}


def test_custom_annotate_df(dataframe1, dataframe2):
    def _annotate_fn(df, uri, **kwargs):
        uristr = os.path.basename(uri)
        df['sample'] = uristr.split(".")[0]
        return df

    df1 = odo(str(dataframe1), DataFrame, annotate=True,
              annotation_fn=_annotate_fn)
    df2 = df1.append(odo(str(dataframe2), DataFrame, annotate=True,
                         annotation_fn=_annotate_fn))
    assert set(df2['sample']) == {'sample1', 'sample2'}


def test_annotate_df_regex(dataframe1, dataframe2):
    df1 = odo(str(dataframe1), DataFrame, annotate=True,
              regex=".*(?P<sample>sample\d+)\.(?P<df>[a-z]+\d+)\.csv")
    df2 = df1.append(
        odo(str(dataframe2), DataFrame,
            annotate=True,
            regex=".*(?P<sample>sample\d+)\.(?P<df>[a-z]+\d+)\.csv"))
    assert set(df2['sample'] == {'sample1', 'sample2'})
    assert set(df2['df'] == {'dataframe1', 'dataframe2'})


def test_pivot_uri(dataframe1):
    df = odo(str(dataframe1), DataFrame, columns="foo", values="bar",
             index="uri", annotate=True)
    assert df.shape == (1, 2)
    assert list(df.columns) == [1, 3]
    assert df.index.name == "uri"


def test_pivot_regex(dataframe1):
    df = odo(str(dataframe1), DataFrame, columns="foo", values="bar",
             index="sample",
             regex=".*(?P<sample>sample\d+)\.(?P<df>[a-z]+\d+)\.csv")
    assert list(df.index) == ["sample1"]


def test_pivot_no_index(dataframe1):
    with pytest.raises(ValueError):
        odo(str(dataframe1), DataFrame, values="bar",
            annotate=True)


def test_pivot_no_column(dataframe1):
    with pytest.raises(ValueError):
        odo(str(dataframe1), DataFrame, values="bar",
            annotate=True, index="uri")


def test_pivot_no_value(dataframe1):
    df = odo(str(dataframe1), DataFrame, columns="bar",
             annotate=True, index="uri")
    assert df.index.name == "uri"



# Copyright (C) 2016 by Per Unneberg
import logging
import pandas as pd
import bioodo
from bioodo import resource, annotate_by_uri, pivot, DataFrame, utils

logger = logging.getLogger(__name__)
config = bioodo.__RESOURCE_CONFIG__['bcftools']


SECTION_NAMES = ['ID', 'SN', 'TSTV', 'SiS', 'AF', 'QUAL', 'IDD', 'ST', 'DP']
COLUMNS = {
    'ID': ['id', 'filename'],
    'SN': ['id', 'key', 'value'],
    'TSTV': ['id', 'ts', 'tv', 'ts/tv', 'ts_(1st_ALT)',
             'tv_(1st_ALT)', 'ts/tv_(1st_ALT)'],
    'SiS': ['id', 'allele_count', 'number_of_SNPs', 'number_of_transitions',
            'number_of_transversions', 'number_of_indels', 'repeat_consistent',
            'repeat_inconsistent', 'NA'],
    'AF': ['id', 'allele_frequency', 'number_of_SNPs', 'number_of_transitions',
           'number_of_transversions', 'number_of_indels', 'repeat_consistent',
           'repeat_inconsistent', 'NA'],
    'QUAL': ['id', 'Quality', 'number_of_SNPs',
             'number_of_transitions_(1st_ALT)',
             'number_of_transversions_(1st_ALT)', 'number_of_indels'],
    'IDD': ['id', 'length_(deletions_negative)', 'count'],
    'ST': ['id', 'type', 'count'],
    'DP': ['id', 'bin', 'number_of_genotypes', 'fraction_of_genotypes',
           'number_of_sites', 'fraction_of_sites'],
}
INDEX_COLUMN = {
    'ID': None,
    'SN': 1,
    'TSTV': None,
    'SiS': None,
    'AF': None,
    'QUAL': 1,
    'IDD': 1,
    'ST': 1,
    'DP': 1,
}


@resource.register(config['stats']['pattern'],
                   priority=config['stats']['priority'])
@pivot
@annotate_by_uri
def resource_bcftools_stats(uri, key="SN", **kwargs):
    """Parse bcftools stats text output file.

    Args:
      uri (str): filename
      key (str): result section to return

    Returns:
      DataFrame: DataFrame for requested section
    """
    if key not in SECTION_NAMES:
        raise KeyError("Not in allowed section names; " +
                       "allowed values are {}".format(
                           ", ".join(SECTION_NAMES + ["Summary"])))
    with open(uri) as fh:
        data = [[y for y in x.replace(":", "").strip().split("\t")[1:]
                 if not y.startswith("#")]
                for x in fh.readlines() if x.startswith(key)]
    df = DataFrame.from_records(data, columns=COLUMNS[key])
    df = df.apply(pd.to_numeric, errors='ignore')
    i = INDEX_COLUMN[key]
    if i is not None:
        df = df.set_index(df[COLUMNS[key][i]])
        del df[COLUMNS[key][i]]
    return df


aggregate = utils.aggregate_factory("bcftools")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from pytest_ngsfixtures.config import application_fixtures
import utils


fixtures = application_fixtures(application="bwa")
bwa_data = utils.fixture_factory([x for x in fixtures])
bwa_aggregate_data = utils.aggregation_fixture_factory(
    [x for x in fixtures], 2)


def test_bwa(bwa_data):
    pass



# Copyright (C) 2015 by Per Unneberg
import logging
import re
import pandas as pd
from functools import wraps
from blaze import append, DataFrame

logger = logging.getLogger(__name__)


@append.register(DataFrame, DataFrame)
def append_dataframe_to_dataframe(tgt, src, **kw):
    tgt = pd.concat([tgt, src])
    return tgt


def annotate_by_uri(func):
    """Decorator function annotate_by_uri.

    Wrap function with wrapper that will optionally add an annotation
    to the data frame based on the uri. This can be particularly
    useful when merging several files and the data provenance needs to
    be tracked.

    Example:

      The following example parses a qualimap output file and
      annotates the resulting data frame with the uri:

      .. code-block:: python

         from bioodo import qualimap, odo
         df = odo("Sample1/genome_results.txt", annotate=True)

      The data frame will contain an extra column named "uri" with the
      value "Sample1/genome_results.txt".

      Alternatively, we can use a regular expression to parse the uri:

      .. code-block:: python

         from bioodo import qualimap, odo
         df = odo("Sample1/genome_results.txt",
                  regex="(?P<sample>Sample[0-9]+)/(?P<file>.*)")

      Here, the data frame will contain two extra columns, "sample"
      with value "Sample1", and "file" with value "genome_results.txt".


    Args:
      annotate (bool): whether or not to annotate data frame
      annotation_fn (function): use custom function to annotate data frame
      regex (str): use regular expression to annotate data frame

    """
    def default_annotation_fn(df, uri, **kwargs):
        logger.debug("Annotating dataframe with uri {}".format(uri))
        df['uri'] = uri

    def regex_annotation_fn(df, uri, regex, **kwargs):
        logger.debug("Searching uri {} with regex {}".format(uri, regex))
        m = re.search(regex, uri)
        if m:
            logger.debug("adding columns {}".format(
                ",".join(["{}={}".format(k, v)
                          for k, v in m.groupdict().items()])))
            for k, v in m.groupdict().items():
                df[k] = v

    @wraps(func)
    def wrap(uri, **kwargs):
        df = func(uri, **kwargs)
        regex = kwargs.get('regex', None)
        annotation_fn = kwargs.get('annotation_fn', None)
        annotate_default = regex is not None or annotation_fn is not None
        annotate = kwargs.get('annotate', annotate_default)
        if not annotate:
            return df
        if regex is not None:
            annotation_fn = regex_annotation_fn
        else:
            annotation_fn = kwargs.get('annotation_fn', default_annotation_fn)
        annotation_fn(df, uri, **kwargs)
        return df
    return wrap


def pivot(func):
    """Decorator function pivot.

    Wrap function with wrapper that will pivot a data frame to wide
    format.

    Example:

      The following example parses a qualimap output file and
      annotates the resulting data frame with a regex, at the same
      time pivoting the data frame so that the sample column is used
      to construct the new index.

      .. code-block:: python

         from bioodo import qualimap, odo
         df = odo("Sample1/genome_results.txt",
                  regex=="(?P<sample>Sample[0-9]+)/(?P<file>.*)",
                  index="sample",
                  columns="statistic",
                  values=["value"])


    The arguments are identical to those in
    :meth:`pandas.DataFrame.pivot`.

    Args:
      index (str): Column name to use to make new frame’s index. If
                   None, uses existing index.
      columns (str): Column name to use to make new frame’s columns
      values (str, list): string or list. Column name to use for
                          populating new frame’s values. If not
                          specified, all remaining columns will be
                          used and the result will have hierarchically
                          indexed columns

    """
    @wraps(func)
    def wrap(uri, **kwargs):
        df = func(uri, **kwargs)
        columns = kwargs.get('columns', None)
        index = kwargs.get('index', None)
        values = kwargs.get('values', None)
        if columns is None and index is None and values is None:
            return df
        if isinstance(values, list):
            dfdict = {k: df.pivot(index=index, columns=columns, values=k)
                      for k in values}
            return pd.concat(dfdict.values(), axis=1, keys=dfdict.keys())
        else:
            return df.pivot(index=index, columns=columns, values=values)
    return wrap



# Copyright (C) 2015 by Per Unneberg
import re
import pandas as pd
import bioodo
from bioodo import resource, annotate_by_uri, pivot, DataFrame, utils
import logging


logger = logging.getLogger(__name__)
config = bioodo.__RESOURCE_CONFIG__['rseqc']


@resource.register(config['read_distribution']['pattern'],
                   priority=config['read_distribution']['priority'])
@pivot
@annotate_by_uri
def resource_read_distribution(uri, **kwargs):
    df = pd.read_table(uri, skiprows=list(range(0, 4)) + [15],
                       sep="[ ]+", engine="python")
    df = df.set_index("Group")
    return df


@resource.register(config['data_frame']['pattern'],
                   priority=config['data_frame']['priority'])
@pivot
@annotate_by_uri
def resource_data_frame(uri, **kwargs):
    df = pd.read_table(uri, header=0)
    return df


@resource.register(config['clipping_profile']['pattern'],
                   priority=config['clipping_profile']['priority'])
@pivot
@annotate_by_uri
def resource_clipping_profile(uri, **kwargs):
    pe = False
    with open(uri) as fh:
        data = "".join(fh)
    header, read = [x.strip() for x in data.split("\n")[0:2]]
    if read == "Read-1:":
        pe = True
    if pe:
        sections = re.split("\nRead-2:\n", data)
        df1 = DataFrame.from_records(
            [x.split("\t") for x in sections[0].split("\n")[2:]])
        df1.columns = header.split("\t")
        df1["Read"] = 1
        df2 = DataFrame.from_records(
            [x.split("\t") for x in sections[1].split("\n")])
        df2.columns = header.split("\t")
        df2["Read"] = 2
        df = pd.concat([df1, df2])
    else:
        df = pd.read_table(uri)
    df.set_index("Position")
    return df


@resource.register(config['xls']['pattern'],
                   priority=config['xls']['priority'])
@pivot
@annotate_by_uri
def resource_xls(uri, **kwargs):
    df = pd.read_table(uri, header=0)
    return df


aggregate = utils.aggregate_factory("rseqc")



# Copyright (C) 2015 by Per Unneberg
from bioodo import star, odo, DataFrame
from pytest_ngsfixtures.config import application_fixtures
import utils


fixtures = application_fixtures(application="star")
data = utils.fixture_factory(fixtures)
aggregate_data = utils.aggregation_fixture_factory(
    [tuple([x[0], x[1], x[2], x[3], {'final': x[4]['final']}])
     for x in fixtures], 2)


def test_star_final_log(data):
    module, command, version, end, pdir = data
    fn = pdir.join("medium.Log.final.out")
    df = odo(str(fn), DataFrame)
    assert df.loc["Number of input reads", "value"] == 30483


def test_star_aggregate(aggregate_data):
    module, command, version, end, pdir = aggregate_data
    df = star.aggregate([str(x.listdir()[0]) for x in pdir.listdir()
                         if x.isdir()],
                        regex=".*/(?P<repeat>[0-9]+)/medium.Log.final.out")
    assert sorted(list(df["repeat"].unique())) == ['0', '1']



# Copyright (C) 2015 by Per Unneberg
from bioodo import sga, odo, DataFrame
from pytest_ngsfixtures.config import application_fixtures
import utils

fixtures = application_fixtures(application="sga")

sga_preprocess_data = utils.fixture_factory([x for x in fixtures if
                                             "preprocess" in x[1]])
sga_filter_data = utils.fixture_factory([x for x in fixtures if
                                         "filter" in x[1]])
sga_aggregate_filter_data = utils.aggregation_fixture_factory(
    [x for x in fixtures if "filter" in x[1]], 2)


def test_sga_preprocess(sga_preprocess_data):
    module, command, version, end, pdir = sga_preprocess_data
    df = odo(str(pdir.listdir()[0]), DataFrame)
    n = 10000 if end == "se" else 20000
    assert df.loc["Reads parsed", "value"] == n


def test_sga_filter(sga_filter_data):
    _filter_stats = {'0.10.13': {'se': 9400, 'pe': 16670}}
    module, command, version, end, pdir = sga_filter_data
    df = odo(str(pdir.listdir()[0]), DataFrame)
    assert (df.loc["Reads failed kmer check", "value"] ==
            _filter_stats[version][end])


def test_sga_aggregate_filter(sga_aggregate_filter_data):
    module, command, version, end, pdir = sga_aggregate_filter_data
    df = sga.aggregate(
        [str(x.listdir()[0]) for x in pdir.listdir() if x.isdir()],
        regex=".*/(?P<repeat>[0-9]+)/sga.filter.log",
        parser=sga.resource_sga_filter)
    assert sorted(list(df["repeat"].unique())) == ['0', '1']



# Copyright (C) 2015 by Per Unneberg
import pandas as pd
import bioodo
from bioodo import resource, annotate_by_uri, pivot, utils

config = bioodo.__RESOURCE_CONFIG__['rsem']


@resource.register(config['genes']['pattern'],
                   priority=config['genes']['priority'])
@pivot
@annotate_by_uri
def resource_genes_results(uri, **kwargs):
    with open(uri):
        data = pd.read_csv(uri, sep="\t", header=0, comment="#",
                           index_col=["gene_id"])
    return data


@resource.register(config['isoforms']['pattern'],
                   priority=config['isoforms']['priority'])
@pivot
@annotate_by_uri
def resource_isoforms_results(uri, **kwargs):
    with open(uri):
        data = pd.read_csv(uri, sep="\t", header=0, comment="#",
                           index_col=["transcript_id"])
    return data


aggregate = utils.aggregate_factory("rsem")



# Copyright (C) 2016 by Per Unneberg
import re
import logging
import pandas as pd
from bioodo import resource, annotate_by_uri, pivot, DataFrame, utils
import bioodo

config = bioodo.__RESOURCE_CONFIG__['vsearch']
logger = logging.getLogger(__name__)

SECTION_NAMES = ["Read length distribution", "Q score distribution",
                 "Truncate at first Q"]


@resource.register(config['stats']['pattern'],
                   priority=config['stats']['priority'])
@pivot
@annotate_by_uri
def resource_vsearch_fastqc_stats(uri, key="Read length distribution",
                                  **kwargs):
    """Parse vsearch fastqc_stats text output file.

    Args:
      uri (str): filename
      key (str): result section to return

    Returns:
      DataFrame: DataFrame for requested section
    """
    if key not in SECTION_NAMES:
        raise KeyError("Not in allowed section names; allowed " +
                       "values are {}".format(", ".join(SECTION_NAMES)))
    with open(uri) as fh:
        data = "".join(fh.readlines())
    if key == "Read length distribution":
        m = re.search(
            "Read length distribution\n(?P<header>[a-zA-Z\s]+)\n" +
            "[\s\-]+\n(?P<data>.*)Q score",
            data, re.DOTALL)
        indexcol = "L"
    elif key == "Q score distribution":
        m = re.search(
            "Q score distribution\s*\n(?P<header>[a-zA-Z\s]+)\n" +
            "[\s\-]+\n(?P<data>.*)\n\s+L\s+PctRecs",
            data, re.DOTALL)
        indexcol = "Q"
    elif key == "Truncate at first Q":
        m = re.search(
            "Truncate at first Q\s*\n(?P<header>[0-9=a-zA-Z\s]+)\n" +
            "[\s\-]+\n(?P<data>.*)\n\n\s+\d+\s+Recs",
            data, re.DOTALL)
        indexcol = "Len"
    else:
        logger.warn("No such key '{}'".format(key))
    try:
        header = re.split("\s+", m.group("header").strip())
        if key == "Read length distribution":
            d = [re.split("\s+", re.sub("[><=]+", "", x).strip())
                 for x in re.split("\n", m.group("data").strip())]
        else:
            d = [re.split("\s+", x.strip())
                 for x in re.split("\n", m.group("data").strip())]
        df = DataFrame.from_records(d, columns=header)
        df = df.apply(pd.to_numeric, errors='ignore')
        df = df.set_index(indexcol)
    except:
        raise
    return df


aggregate = utils.aggregate_factory("vsearch")



# Copyright (C) 2015 by Per Unneberg
import re
import pandas as pd
import logging
import bioodo
from bioodo import resource, annotate_by_uri, pivot, DataFrame, utils

logger = logging.getLogger(__name__)
config = bioodo.__RESOURCE_CONFIG__['cutadapt']


# Potentially add regexp for adapter sections as these are repetitive
regex = "\n===\s*(?P<Read>(First read|Second read)?):?" + \
        "\s+Adapter\s+'(?P<Adapter>[^\s]+)'\s+==="
adapter_re = re.compile(regex)


re_trim = re.compile(r'(\([0-9.]+%\)|,| |bp)')


def _split_x(x, delim=":"):
    y = x.strip().split(delim)
    return [y[0], re_trim.sub("", y[1])]


# For now only return the summary section
@resource.register(config['metrics']['pattern'],
                   priority=config['metrics']['priority'])
@pivot
@annotate_by_uri
def resource_cutadapt_metrics(uri, **kwargs):
    with open(uri) as fh:
        data = "".join(fh)
    sections = re.split("\n===.*===\n", data)
    df = DataFrame.from_records(
        [_split_x(x) for x in sections[1].split("\n") if x],
        index=["statistic"], columns=["statistic", "value"])
    df["value"] = pd.to_numeric(df["value"])
    return df


aggregate = utils.aggregate_factory("cutadapt")



from __future__ import print_function

# stdlib
import os
from setuptools import setup
from os.path import realpath, dirname, relpath, join

# Extensions
import versioneer

# --------------------------------------------------
# globals and constants
# --------------------------------------------------

ROOT = dirname(realpath(__file__))

# --------------------------------------------------
# classes and functions
# --------------------------------------------------

package_data = []

def package_path(path, filters=()):
    if not os.path.exists(path):
        raise RuntimeError("packaging non-existent path: %s" % path)
    elif os.path.isfile(path):
        package_data.append(relpath(path, 'bioodo'))
    else:
        for path, dirs, files in os.walk(path):
            path = relpath(path, 'bioodo')
            for f in files:
                if not filters or f.endswith(filters):
                    package_data.append(join(path, f))

package_path(join(ROOT, 'bioodo', 'data'), 'config.yaml')

scripts = []                    

REQUIRES = [
    'pandas',
    'odo',
    'blaze',
]
                    
_version = versioneer.get_version()
_cmdclass = versioneer.get_cmdclass()

setup(
    name="bioodo",
    version=_version,
    cmdclass=_cmdclass,
    author="Per Unneberg",
    author_email="per.unneberg@scilifelab.se",
    description="odo extensions for use with bioinformatics",
    license="MIT",
    url="http://github.com/percyfal/bioodo",
    scripts=scripts,
    packages=[
        'bioodo',
        'bioodo.tests',
    ],
    package_data={'bioodo': package_data},
    install_requires=REQUIRES,
    test_requires=["pytest"],
)



# Copyright (C) 2015 by Per Unneberg
import re
import logging
import math
from datetime import datetime
import pandas as pd
from blaze import odo, DataFrame

logger = logging.getLogger(__name__)


# FIXME: utilize pandas builtin functionality for handling these issues
def recast(x, strpfmt="%b %d %H:%M:%S"):
    """Reformat strings to numeric or datestrings.

    Args:
      x (str): string
      strpfmt (str): strpfmt time string

    Returns:
      datetime.strptime: datetime object
    """
    if isinstance(x, float):
        if math.isnan(x):
            return x
    x = x.rstrip().lstrip()
    if re.match('^[0-9]+$', x):
        return int(x)
    elif re.match('^[0-9]+[,\.][0-9]+$', x):
        return float(x.replace(",", "."))
    elif re.search("%", x):
        return float(x.replace(",", ".").replace("%", ""))
    else:
        try:
            dateobj = datetime.strptime(x, strpfmt)
            return dateobj
        except:
            return x


# Replace whitespace with underscore, convert percent characters to PCT
def trim_header(x, underscore=False, percent=False):
    y = x.lstrip().rstrip()
    y = y.replace(" ", "_" if underscore else " ")
    y = y.replace(",", "_" if underscore else " ")
    if percent:
        y = y.replace("%", "PCT")
    return y


def annotate_df(infile, parser, groupnames=["SM"]):
    """Annotate a parsed odo unit.

    Assumes metadata information is stored in input file name.

    Args:
      infile (str): file name
      parser (re): regexp object to parse input file name with.
                   Metadata information to parse is stored in file name

      groupnames (list): list of parser group names to use. For each
                         name <name>, the parser should have a
                         corresponding (?P<name>...) expression

    """
    df = odo(infile, pd.DataFrame)
    m = parser.parse(infile)
    for name in groupnames:
        df[name] = str(m[name])
    return df


def aggregate_factory(module):
    """Factory function to generate aggregation function.

    Args:
      module (str): module name

    Returns:
      function


    Given a list of input files for a bioinformatics application,
    seamlessly apply odo to each input file and concatenate the
    results. Additional file-based information, such as sample names,
    can be added on the fly by supplying a regular expression with
    group names that parses the file names. The regular expression
    group name will be added to the data frame.


    Examples:

      The following example uses :func:`bioodo.qualimap.aggregate` to
      parse two qualimap output files, extract sample names via the
      regular expression, and concatenate the results together with an
      additional column named "sample".

      .. code-block:: python

         from bioodo import qualimap
         qualimap_files = ["Sample1/genome_results.txt",
                           "Sample2/genome_results.txt"]
         df = qualimap.aggregate(qualimap_files,
                                 regex="(?P<sample>Sample[0-9]+)/genome_results.txt")

    Args:
      infiles (list): list of input file names
      regex (str): regex pattern to parse file
      parser (func): bioodo parser function to use in case the generic
                     parsing fails
      outfile (str): outfile name. Compression will be inferred from suffix
      kwargs (dict): keyword arguments

    Returns:
      Aggregated data frame or None if outfile given
    """
    def aggregate(infiles, parser=None, outfile=None, **kwargs):
        """Helper function to aggregate files

        Given a list of input files for a bioinformatics application,
        seamlessly apply odo to each input file and concatenate the
        results. Additional file-based information, such as sample names,
        can be added on the fly by supplying a regular expression with
        group names that parses the file names. The regular expression
        group name will be added to the data frame.

        Note that the function is not directly called from
        :mod:`bioodo.utils` but rather from one of the application
        modules, as the following example shows.

        Examples:

          The following example uses :func:`bioodo.qualimap.aggregate` to
          parse two qualimap output files, extract sample names via the
          regular expression, and concatenate the results together with an
          additional column named "sample".

          .. code-block:: python

             from bioodo import qualimap
             qualimap_files = ["Sample1/genome_results.txt",
                               "Sample2/genome_results.txt"]
             df = qualimap.aggregate(qualimap_files,
                                 regex="(?P<sample>Sample[0-9]+)/genome_results.txt")

        Args:
          infiles (list): list of input file names
          parser (func): bioodo parser function to use in case the generic
                         parsing fails
          outfile (str): outfile name. Compression will be inferred from suffix
          regex (str): regex pattern to parse file
          kwargs (dict): keyword arguments
          long (bool): output data in long format (default False)

        See also arguments to :func:`pandas.annotate_by_uri` for more
        annotation options.

        Returns:
          Aggregated data frame or None if outfile given

        """
        logger.debug(
            "Aggregating {module} infiles".format(module=module) +
            " {infiles} in {module} aggregate".format(
                module=module, infiles=",".join(infiles)))
        compression = None
        if outfile:
            _map = {'gz': 'gzip', 'bz2': 'bz2', 'xz': 'xz'}
            m = re.search("\.(?P<compression>gz|bz2|xz)$", outfile)
            if m:
                compression = _map[m.group("compression")]
        import odo
        dflist = []
        for f in infiles:
            logger.debug("loading {}".format(f))
            if parser:
                df = odo.odo(parser(f, **kwargs), DataFrame)
            else:
                try:
                    df = odo.odo(f, DataFrame, **kwargs)
                except NotImplementedError:
                    logger.error("Unable to parse uri {uri};".format(uri=f) +
                                 " check that file extension is handled by" +
                                 " the {module}".format(module=module) +
                                 " bioodo module;" +
                                 " else configure extension in .bioodo.yaml")
                    raise
            dflist.append(df)
        df = pd.concat(dflist)
        if outfile:
            df.to_csv(outfile, compression=compression)
        else:
            return df
    return aggregate



# Copyright (C) 2016 by Per Unneberg
import re
from os.path import join, basename
import logging
import bioodo
from bioodo import resource, annotate_by_uri, pivot, DataFrame, utils


logger = logging.getLogger(__name__)
config = bioodo.__RESOURCE_CONFIG__['fastqc']


# Possible section names
SECTION_NAMES = ['Header', 'Basic_Statistics',
                 'Per_base_sequence_quality', 'Per_tile_sequence_quality',
                 'Per_sequence_quality_scores', 'Per_base_sequence_content',
                 'Per_sequence_GC_content', 'Per_base_N_content',
                 'Sequence_Length_Distribution', 'Sequence_Duplication_Levels',
                 'Overrepresented_sequences', 'Adapter_Content',
                 'Kmer_Content']


@resource.register(config['fastqc']['pattern'],
                   priority=config['fastqc']['priority'])
@pivot
@annotate_by_uri
def resource_fastqc_data(uri, key="Basic_Statistics", **kwargs):
    """Parse fastqc text output file.

    Args:
      uri (str): filename
      key (str): result section to return

    Returns:
      DataFrame: DataFrame for requested section
    """
    def _parse():
        fastqc_data = kwargs.get('fastqc_data', 'fastqc_data.txt')
        if uri.endswith(".zip"):
            logger.debug("Reading zipped fastqc file")
            from zipfile import ZipFile
            with ZipFile(uri) as zf:
                with zf.open(
                        join(basename(uri.strip(".zip")), fastqc_data)) as fh:
                    data = re.sub(">>END_MODULE", "",
                                  fh.read().decode("utf-8"))
        else:
            logger.debug("Reading fastqc_data.txt file")
            with open(uri) as fh:
                data = re.sub(">>END_MODULE", "", "".join(fh))
        return data

    logger.debug("Reading {}".format(uri))
    if key not in SECTION_NAMES + ['Summary']:
        raise KeyError(
            "Not in allowed section names; allowed values are {}".format(
                ", ".join(SECTION_NAMES + ["Summary"])))
    data = _parse()
    sections = [x for x in re.split(">>+[a-zA-Z _\t\n]+", data)]
    headings = ['Header'] + \
               [re.sub(" ", "_", (re.sub(">>", "", x.split("\t")[0])))
                for x in re.split("\n", data) if x.startswith(">>")]
    if key not in headings + ['Summary']:
        raise KeyError("Not in available names; " +
                       "analysis has not been performed")
    # Pass/fail summary
    if key == "Summary":
        return DataFrame.from_records(
            [re.sub(" ", "_", x).split("\t")
             for x in re.findall(">>(.+)", data)],
            columns=["Statistic", "Value"], index="Statistic")
    for h, sec in zip(headings, sections):
        if not h == key:
            continue
        logger.debug("Parsing section {}".format(h))
        if h == "Header":
            d = DataFrame.from_records(
                [[re.sub("#", "", x)
                  for x in re.split("\t", sec.strip())]])
        else:
            i = 1 if h.startswith("Sequence_Duplication") else 0
            columns = [re.sub("#", "", x)
                       for x in re.split("\t", sec.split("\n")[i].strip())]
            d = DataFrame.from_records(
                [re.split("\t", x.strip())
                 for x in sec.split("\n") if x and not x.startswith("#")],
                columns=columns, index=columns[0])
    return d


aggregate = utils.aggregate_factory("fastqc")



# Copyright (C) 2015 by Per Unneberg
from bioodo import bcftools, odo, DataFrame
from pytest_ngsfixtures.config import application_fixtures
import utils

fixtures = application_fixtures(application="bcftools")

stat_fixtures = [x for x in fixtures if x[1] == "bcftools_stats"]
bcftools_stats = utils.fixture_factory(stat_fixtures)
bcftools_aggregate_QUAL_data = utils.aggregation_fixture_factory(
    [x for x in stat_fixtures], 2)


def test_basic_statistics(bcftools_stats):
    module, command, version, end, pdir = bcftools_stats
    fn = str(pdir.join("medium.call.stats"))
    df = odo(fn, DataFrame)
    assert (list(df.index)[0] == 'number of samples')
    n = 10667 if end == "pe" else 7400
    assert(df.loc["number of records", "value"] == n)


def test_TSTV(bcftools_stats):
    module, command, version, end, pdir = bcftools_stats
    fn = str(pdir.join("medium.call.stats"))
    df = odo(fn, DataFrame, key="TSTV")
    tstv = 2.12 if end == "pe" else 2.19
    assert (df.loc[0]["ts/tv"] == tstv)


def test_IDD(bcftools_stats):
    module, command, version, end, pdir = bcftools_stats
    fn = str(pdir.join("medium.call.stats"))
    df = odo(fn, DataFrame, key="IDD")
    count = 123 if end == "pe" else 95
    assert (df.loc[-1]["count"] == count)


def test_QUAL(bcftools_stats):
    module, command, version, end, pdir = bcftools_stats
    fn = str(pdir.join("medium.call.stats"))
    df = odo(fn, DataFrame, key="QUAL")
    assert "number_of_transitions_(1st_ALT)" in list(df.columns)
    nsnps = 83 if end == "pe" else 90
    assert (df.loc[3]["number_of_SNPs"] == nsnps)


def test_bcftools_aggregate_QUAL(bcftools_aggregate_QUAL_data):
    module, command, version, end, pdir = bcftools_aggregate_QUAL_data
    df = bcftools.aggregate(
        [str(x.listdir()[0]) for x in pdir.listdir() if x.isdir()],
        regex=".*/(?P<repeat>[0-9]+)/medium.call.stats",
        key="QUAL")
    assert df.index.name == "Quality"
    assert sorted(list(df["repeat"].unique())) == ['0', '1']



# Copyright (C) 2015 by Per Unneberg
from blaze import resource, DataFrame
import pandas as pd
from .pandas import annotate_by_uri, pivot
import logging
import bioodo
from bioodo import utils


config = bioodo.__RESOURCE_CONFIG__['picard']
logger = logging.getLogger(__name__)


def _hist_reader(uri):
    with open(uri) as fh:
        data = [x.strip("\n").split("\t") for x in fh
                if not x.strip() == ""]
        indices = list((i for i, val in enumerate(data)
                        if val[0].startswith("## METRICS CLASS") or
                        val[0].startswith("## HISTOGRAM")))
        if len(indices) == 1:
            indices.append(len(data))
        metrics = DataFrame.from_records(data[(indices[0] + 2):(indices[1])],
                                         columns=data[(indices[0] + 1)])
        # We could be missing the histogram
        try:
            hist = DataFrame.from_records(data[(indices[1] + 2):],
                                          columns=data[(indices[1] + 1)])
        except:
            logger.warn("No histogram data for {}".format(uri))
            hist = None
    return (metrics, hist)


def _reader(uri):
    with open(uri) as fh:
        data = [x.strip("\n").split("\t") for x in fh
                if not x.strip() == ""]
        indices = list((i for i, val in enumerate(data)
                        if val[0].startswith("## METRICS CLASS")))
        metrics = DataFrame.from_records(data[(indices[0] + 2):],
                                         columns=data[(indices[0] + 1)],
                                         index="CATEGORY")
    return (metrics, None)


@resource.register(config['align_metrics']['pattern'],
                   priority=config['align_metrics']['priority'])
@pivot
@annotate_by_uri
def resource_align_metrics(uri, **kwargs):
    """Parse picard AlignmentSummaryMetrics text output file.

    Args:
      uri (str): filename

    Returns:
      DataFrame
    """
    metrics, _ = _reader(uri)
    metrics = metrics.apply(pd.to_numeric, axis=1)
    return metrics


@resource.register(config['insert_metrics']['pattern'],
                   priority=config['insert_metrics']['priority'])
@pivot
@annotate_by_uri
def resource_insert_metrics(uri, key="metrics", **kwargs):
    """Parse picard InsertSummaryMetrics text output file.

    Args:
      uri (str): filename
      key (str): result section to return (hist or metrics)

    Returns:
      DataFrame for requested section
    """
    logger.debug("Reading {}".format(uri))
    (_metrics, hist) = _hist_reader(uri)
    metrics = _metrics[_metrics.columns.difference(
        ["PAIR_ORIENTATION"])].apply(pd.to_numeric, axis=0)
    metrics["PAIR_ORIENTATION"] = _metrics["PAIR_ORIENTATION"]
    hist = hist.apply(pd.to_numeric, axis=0)
    if key == "metrics":
        return metrics
    elif key == "hist":
        return hist


@resource.register(config['hs_metrics']['pattern'],
                   priority=config['hs_metrics']['priority'])
@pivot
@annotate_by_uri
def resource_hs_metrics(uri, **kwargs):
    """Parse picard HybridSelectionMetrics text output file.

    Args:
      uri (str): filename

    Returns:
      DataFrame
    """
    return _hist_reader(uri)


@resource.register(config['dup_metrics']['pattern'],
                   priority=config['dup_metrics']['priority'])
@pivot
@annotate_by_uri
def resource_dup_metrics(uri, key="metrics", **kwargs):
    """Parse picard DuplicationMetrics text output file.

    Args:
      uri (str): filename
      key (str): result section to return (hist or metrics)

    Returns:
      DataFrame for requested section
    """
    (_metrics, hist) = _hist_reader(uri)
    metrics = _metrics[
        _metrics.columns.difference(["LIBRARY"])].apply(pd.to_numeric, axis=0)
    if hist is not None:
        hist = hist.apply(pd.to_numeric, axis=0)
    if key == "metrics":
        return metrics
    elif key == "hist":
        return hist


aggregate = utils.aggregate_factory("picard")



# Copyright (C) 2015 by Per Unneberg
import pandas as pd
import bioodo
from bioodo import resource, annotate_by_uri, pivot, utils
import logging


logger = logging.getLogger(__name__)
config = bioodo.__RESOURCE_CONFIG__['star']


@resource.register(config['log_final']['pattern'],
                   priority=config['log_final']['priority'])
@pivot
@annotate_by_uri
def resource_star_log(uri, **kwargs):
    """Parse Star Log.final.out log file"""
    df = pd.read_table(uri, sep="|", names=["name", "value"])
    df["name"] = [x.strip() for x in df["name"]]
    df["value"] = [utils.recast(x) for x in df["value"]]
    df = df.set_index("name")
    return df


aggregate = utils.aggregate_factory("star")



# Copyright (C) 2015 by Per Unneberg
from bioodo import mapdamage, odo, DataFrame
from pytest_ngsfixtures.config import application_fixtures
import utils

fixtures = application_fixtures(application="mapdamage2")
mapdamage_data = utils.fixture_factory(fixtures)
mapdamage_agg_data_misincorp = utils.aggregation_fixture_factory(
    [x for x in fixtures], 2, keys=["misincorp"])


def test_mapdamage_runtime(mapdamage_data):
    module, command, version, end, pdir = mapdamage_data
    fn = pdir.join("Runtime_log.txt")
    odo(str(fn), DataFrame)


def test_mapdamage_3pGtoA(mapdamage_data):
    module, command, version, end, pdir = mapdamage_data
    fn = pdir.join("3pGtoA_freq.txt")
    df = odo(str(fn), DataFrame)
    assert(df.index.name == "pos")


def test_mapdamage_5pCtoT(mapdamage_data):
    module, command, version, end, pdir = mapdamage_data
    fn = pdir.join("5pCtoT_freq.txt")
    df = odo(str(fn), DataFrame)
    assert(df.index.name == "pos")


def test_mapdamage_mcmc_correct_prob(mapdamage_data):
    module, command, version, end, pdir = mapdamage_data
    fn = pdir.join("Stats_out_MCMC_correct_prob.csv")
    df = odo(str(fn), DataFrame)
    assert(df.index.name == "Position")
    assert(df.shape[1] == 2)


def test_mapdamage_mcmc_iter(mapdamage_data):
    module, command, version, end, pdir = mapdamage_data
    fn = pdir.join("Stats_out_MCMC_iter.csv")
    df = odo(str(fn), DataFrame)
    assert(df.shape[1] == 6)


def test_mapdamage_mcmc_iter_summ(mapdamage_data):
    module, command, version, end, pdir = mapdamage_data
    fn = pdir.join("Stats_out_MCMC_iter_summ_stat.csv")
    df = odo(str(fn), DataFrame)
    assert(df.shape[1] == 6)


def test_mapdamage_dnacomp(mapdamage_data):
    module, command, version, end, pdir = mapdamage_data
    fn = pdir.join("dnacomp.txt")
    df = odo(str(fn), DataFrame)
    assert (df["Chr"][0] == "scaffold1")


def test_mapdamage_dnacomp_genome(mapdamage_data):
    module, command, version, end, pdir = mapdamage_data
    fn = pdir.join("dnacomp_genome.csv")
    df = odo(str(fn), DataFrame)
    assert (list(df["A"])[0] - 0.265786 < 0.0001)


def test_mapdamage_lgdistribution(mapdamage_data):
    module, command, version, end, pdir = mapdamage_data
    fn = pdir.join("lgdistribution.txt")
    df = odo(str(fn), DataFrame)
    assert (list(df.columns) == ['Std', 'Length', 'Occurences'])


def test_mapdamage_misincorporation(mapdamage_data):
    module, command, version, end, pdir = mapdamage_data
    fn = pdir.join("misincorporation.txt")
    df = odo(str(fn), DataFrame)
    assert (df.shape[1] == 30)


def test_mapdamage_aggregate_misincorporation(mapdamage_agg_data_misincorp):
    module, command, version, end, pdir = mapdamage_agg_data_misincorp
    df = mapdamage.aggregate(
        [str(x.listdir()[0]) for x in pdir.listdir() if x.isdir()],
        regex=".*/(?P<repeat>[0-9]+)/misincorporation.txt")
    assert sorted(list(df["repeat"].unique())) == ['0', '1']



import os
import yaml
from bioodo import settings

__import__('pkg_resources').declare_namespace(__name__)

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

ROOT = os.path.dirname(os.path.realpath(__file__))

# Make odo visible via bioodo
from odo import odo

# Module imports for all submodules
from blaze import resource, DataFrame
from .pandas import annotate_by_uri, pivot
import numpy as np
import re
import logging
from . import config

logger = logging.getLogger(__name__)


# Set global configuration varibable to hold resource configuration
global __RESOURCE_CONFIG__
with open(os.path.join(ROOT, "data", "config.yaml")) as fh:
    __RESOURCE_CONFIG__ = yaml.load(fh)

def load_config():
    for f in settings.CONFIGFILES:
        logger.info("Looking for configuration file {}".format(f))
        logger.debug("Looking for configuration file {}".format(f))
        if os.path.exists(f):
            logger.debug("Reading local config {}".format(f))
            with open(f) as fh:
                _localconfig = yaml.load(fh)
            config.update_config(__RESOURCE_CONFIG__, _localconfig)

load_config()
